/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase;

import ru.ispras.testbase.generator.DataGenerator;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static ru.ispras.fortress.util.InvariantChecks.checkNotNull;

public final class TestBaseRegistry {
  private static final class GeneratorRegistry {
    final Map<String, List<DataGenerator>> registry = new HashMap<>();
  
    public boolean add(final String name, final DataGenerator generator) {
      checkNotNull(name);
      checkNotNull(generator);
  
      List<DataGenerator> list = registry.get(name);
      if (list == null) {
        list = new ArrayList<>();
        list.add(generator);
        registry.put(name, list);
      } else {
        for (final DataGenerator stored : list) {
          if (stored.equals(generator)) {
            return false;
          }
        }
        list.add(generator);
      }
      return true;
    }
  
    public List<DataGenerator> getNamed(final String name) {
      checkNotNull(name);
      final List<DataGenerator> list = registry.get(name);
      if (list == null) {
        return Collections.emptyList();
      }
      return Collections.unmodifiableList(list);
    }
  }

  private final GeneratorRegistry generators = new GeneratorRegistry();

  public boolean registerGenerator(final String name, final DataGenerator generator) {
    return generators.add(name, generator);
  }

  public Collection<DataGenerator> getNamedGenerators(final String name) {
    return generators.getNamed(name);
  }
}
