/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.generator;

import ru.ispras.testbase.TestBaseQuery;
import ru.ispras.testbase.TestDataProvider;

/**
 * The {@link DataGenerator} interface is to be supported by all data generators.
 * 
 * @author <a href="mailto:andrewt@ispras.ru">Andrei Tatarnikov</a>
 */
public interface DataGenerator {
  /**
   * Checks whether the data generator is capable of generating data
   * for the specified query.
   * 
   * @param query Query to be processed.
   * @return {@code true} if the query can be handled by the generator or
   *         {@code false} otherwise.
   */
  boolean isSuitable(TestBaseQuery query);

  /**
   * Generates data for the specified query.
   * 
   * @param query Query to be processed.
   * @return Test data provider that holds the generated data.
   */
  TestDataProvider generate(TestBaseQuery query);
}
